/**
 * EasyBeans
 * Copyright (C) 2006 Bull S.A.S.
 * Contact: easybeans@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: StatefulBean.java 9 2006-02-19 18:53:32Z benoitf $
 * --------------------------------------------------------------------------
 */

package org.objectweb.easybeans.examples.statefulbean;

import java.rmi.RemoteException;

import javax.ejb.Remote;
import javax.ejb.SessionSynchronization;
import javax.ejb.Stateful;

/**
 * Simple stateful bean.
 * @author Florent Benoit
 */
@Stateful
@Remote(StatefulRemote.class)
public class StatefulBean implements SessionSynchronization, StatefulRemote {

    /**
     * Actual state of the bean.
     */
    private int total = 0;

    /**
     * value inside Tx, not yet committed.
     */
    private int newtotal = 0;

    /**
     * Initialize the amounts for with and without Tx.
     */
    public StatefulBean() {
        total = 0;
        newtotal = total; // in case we are outside transactions
    }

    /**
     * The afterBegin method notifies a session Bean instance that a new
     * transaction has started, and that the subsequent business methods on the
     * instance will be invoked in the context of the transaction. The instance
     * can use this method, for example, to read data from a database and cache
     * the data in the instance fields. This method executes in the proper
     * transaction context.
     * @throws java.rmi.RemoteException - This exception is defined in the
     *         method signature to provide backward compatibility for enterprise
     *         beans written for the EJB 1.0 specification. Enterprise beans
     *         written for the EJB 1.1 and higher specifications should throw
     *         the javax.ejb.EJBException instead of this exception. Enterprise
     *         beans written for the EJB 2.0 and higher specifications must not
     *         throw the java.rmi.RemoteException.
     */
    public void afterBegin() throws RemoteException {
        newtotal = total;
    }

    /**
     * The beforeCompletion method notifies a session Bean instance that a
     * transaction is about to be committed. The instance can use this method,
     * for example, to write any cached data to a database. This method executes
     * in the proper transaction context. <b>Note: </b> The instance may still
     * cause the container to rollback the transaction by invoking the
     * setRollbackOnly() method on the instance context, or by throwing an
     * exception.
     * @throws java.rmi.RemoteException - This exception is defined in the
     *         method signature to provide backward compatibility for enterprise
     *         beans written for the EJB 1.0 specification. Enterprise beans
     *         written for the EJB 1.1 and higher specifications should throw
     *         the javax.ejb.EJBException instead of this exception. Enterprise
     *         beans written for the EJB 2.0 and higher specifications must not
     *         throw the java.rmi.RemoteException.
     */
    public void beforeCompletion() throws RemoteException {
    }

    /**
     * The afterCompletion method notifies a session Bean instance that a
     * transaction commit protocol has completed, and tells the instance whether
     * the transaction has been committed or rolled back. This method executes
     * with no transaction context.
     * @param committed - True if the transaction has been committed, false if
     *        is has been rolled back.
     * @throws java.rmi.RemoteException - This exception is defined in the
     *         method signature to provide backward compatibility for enterprise
     *         beans written for the EJB 1.0 specification. Enterprise beans
     *         written for the EJB 1.1 and higher specifications should throw
     *         the javax.ejb.EJBException instead of this exception. Enterprise
     *         beans written for the EJB 2.0 and higher specifications must not
     *         throw the java.rmi.RemoteException.
     */
    public void afterCompletion(final boolean committed) throws RemoteException {
        if (committed) {
            total = newtotal;
        } else {
            newtotal = total;
        }
    }

    /**
     * Business method implementation.
     * @param s nb of shares to be bought
     */
    public void buy(final int s) {
        newtotal = newtotal + s;
        return;
    }

    /**
     * Business method implementation.
     * @return the nb of shares bought
     */
    public int read() {
        return newtotal;
    }
}
